@@ -101,6 +101,7 @@ db_search(DB *dbp, const char *db_file, char *search_string, char **passwd)
     DBC *dbcp = NULL;
     DBT key, data;
     int ret, t_ret;
+    char *tmp_passwd;
 
     // make cursor to search 
     ret = (dbp)->cursor(dbp, NULL, &dbcp, 0);
@@ -135,7 +136,12 @@ db_search(DB *dbp, const char *db_file, char *search_string, char **passwd)
     }
     // found 
     if (data.size > 0) {
-	*passwd = strndup(data.data, data.size);
+	tmp_passwd = strndup(data.data, data.size);
+        if (!tmp_passwd) {
+            log(ERR_DB, "db_search", "memory error");
+            return(DB_ERROR);
+        }
+	*passwd = tmp_passwd;
     }
 
     // close cursor
