@@ -57,10 +57,10 @@ pthread_mutex_t pw_lock = PTHREAD_MUTEX_INITIALIZER;
  *
  * Rcpt addr of RCTPDB is retrieved.(mail address or FQDN) 
  *
- * DB *dbp              		DBpointer
- * char *dbpath         		DB path string
- * char *searchstr			Search String
- * char *onetime_pass			Random Password
+ * struct search_res *res
+ * config *cfg
+ * char *rcptaddr			Recipient address
+ * char *searchstr			Search string
  * struct rcptinfo **passlist		List encrypted by fixed password
  * struct rcptinfo ** rdmpasslist	List encrypted by random password
  *
@@ -71,8 +71,9 @@ pthread_mutex_t pw_lock = PTHREAD_MUTEX_INITIALIZER;
  *       MALLOC_ERROR   alloc error
  */
 int
-search_rcptstr_bdb(DB *rcpt_dbp, const char *rcptdbpath, char *rcptaddr, char *searchstr, 
-		   char *onetime_pass, struct rcptinfo **passlist, struct rcptinfo **rdmpasslist)
+search_rcptstr_bdb(struct search_res *res, struct config *cfg,
+                   char *rcptaddr, char *searchstr,
+                   struct rcptinfo **passlist, struct rcptinfo **rdmpasslist)
 {
     int ret;
     char *noenc_searchstr = NULL;
@@ -91,7 +92,7 @@ search_rcptstr_bdb(DB *rcpt_dbp, const char *rcptdbpath, char *rcptaddr, char *s
     /* search noenc_searchstr */
     sprintf(noenc_searchstr , "!%s", searchstr);
 
-    ret = search_str_bdb(rcpt_dbp, rcptdbpath,
+    ret = search_str_bdb(res->rcpt_dbp, cfg->cf_rcptdbpath,
                              searchstr, noenc_searchstr, &passwd);
     if (ret == DB_ERROR) {
         if (passwd != NULL) {
@@ -111,7 +112,6 @@ search_rcptstr_bdb(DB *rcpt_dbp, const char *rcptdbpath, char *rcptaddr, char *s
 	return NOT_ENC;
     }
     if (ret == ENC) {
-        DEBUGLOG("Rcpt address found.(%s)", searchstr);
         if (passwd != NULL) {
             /* Add address list */
             /* get now time */
@@ -122,7 +122,8 @@ search_rcptstr_bdb(DB *rcpt_dbp, const char *rcptdbpath, char *rcptaddr, char *s
             free(passwd);
             passwd = NULL;
 
-            if (add_enclist(&passlist, searchstr, rcptaddr, tmppass) != 0) {
+            if (add_enclist(cfg, &passlist, searchstr, rcptaddr, tmppass,
+                            res->ext_dbp, cfg->cf_extensiondbpath) != 0) {
                 free(noenc_searchstr);
                 return MALLOC_ERROR;
             }
@@ -131,7 +132,9 @@ search_rcptstr_bdb(DB *rcpt_dbp, const char *rcptdbpath, char *rcptaddr, char *s
             passwd = NULL;
         } else {
             /* Add rdmpasslist list */
-            if (add_enclist(&rdmpasslist, OTHERKEYWORD, rcptaddr, onetime_pass) != 0) {
+            if (add_enclist(cfg, &rdmpasslist, searchstr, rcptaddr,
+                            res->onetime_pass, res->ext_dbp,
+                            cfg->cf_extensiondbpath) != 0) {
                 free(noenc_searchstr);
                 return MALLOC_ERROR;
             }
@@ -161,6 +164,10 @@ free_search_rcptaddr(struct search_res *res)
 	db_close(res->rcpt_dbp);
     }
 
+    if (res->ext_dbp != NULL) {
+	db_close(res->ext_dbp);
+    }
+
     if (res->ld != NULL) {
 	ldap_unbind_ext_s(res->ld, NULL, NULL);
     }
@@ -248,6 +255,15 @@ search_rcptaddr(struct config *cfg, char *sender, struct rcptaddr *rcptlist, str
         return ERROR;
     }
 
+    if (cfg->cf_extensiondb && cfg->cf_extensiondb[0] != '\0') {
+        /* extension db open */
+        ret = db_open(&res.ext_dbp, cfg->cf_extensiondbpath,  cfg->cf_extensiondbtype);
+        if (ret == DB_ERROR) {
+            free_search_rcptaddr(&res);
+            return ERROR;
+        }
+    }
+
     for (i = 0; (rcptlist + i)->rcpt_addr != NULL; i++) {
         rcptaddr = (rcptlist + i)->rcpt_addr;
         //rcptaddrlen = (rcptlist + i)->rcpt_addr_len;
@@ -456,13 +472,17 @@ search_rcptaddr_ldap(struct search_res *res, struct config *cfg,
         data.pass = NULL;
     
 	/* Add address list */
-        if (add_enclist(&passlist, rcptaddr, rcptaddr, pass) != 0) {
+
+        if (add_enclist(cfg, &passlist, rcptaddr, rcptaddr, pass,
+                        res->ext_dbp, cfg->cf_extensiondbpath) != 0) {
             return ERROR;
         }
         DEBUGLOG("Add keyword to passlist structure.(%s)", rcptaddr);
     } else {
         /* Add rdmpasslist list */
-        if (add_enclist(&rdmpasslist, OTHERKEYWORD, rcptaddr, res->onetime_pass) != 0) {
+        if (add_enclist(cfg, &rdmpasslist, rcptaddr, rcptaddr,
+                        res->onetime_pass, res->ext_dbp,
+                        cfg->cf_extensiondbpath) != 0) {
             return ERROR;
         }
         free(data.addr);
@@ -507,7 +527,8 @@ check_default_policy(int defalt_pass_len, struct search_res *res, struct config
         strftime(pass, BUFSIZE - 1, cfg->cf_defaultpassword, ti);
 
         /* Add address list */
-        if (add_enclist(&passlist, rcptaddr, rcptaddr, pass) != 0) {
+        if (add_enclist(cfg, &passlist, rcptaddr, rcptaddr, pass,
+                        res->ext_dbp, cfg->cf_extensiondbpath) != 0) {
             return MALLOC_ERROR;
         }
 
@@ -516,7 +537,9 @@ check_default_policy(int defalt_pass_len, struct search_res *res, struct config
 	DEBUGLOG("DefaultPolicy YES: DefaultPassword NO.(%s)", rcptaddr);
 
         /* Add rdmpasslist list */
-        if (add_enclist(&rdmpasslist, OTHERKEYWORD, rcptaddr, res->onetime_pass) != 0) {
+        if (add_enclist(cfg, &rdmpasslist, rcptaddr, rcptaddr,
+                        res->onetime_pass, res->ext_dbp,
+                        cfg->cf_extensiondbpath) != 0) {
             return MALLOC_ERROR;
         }
 
@@ -562,8 +585,8 @@ search_rcptaddr_bdb(struct search_res *res, struct config *cfg, char *rcptaddr,
         return MALLOC_ERROR;
     }
 
-    ret = search_rcptstr_bdb(res->rcpt_dbp, cfg->cf_rcptdbpath, rcptaddr, searchaddr,
-			     res->onetime_pass, passlist, rdmpasslist);
+    ret = search_rcptstr_bdb(res, cfg, rcptaddr, searchaddr,
+                             passlist, rdmpasslist);
     if (ret < 0) {
         free(searchaddr);
         return DB_ERROR;
@@ -579,8 +602,7 @@ search_rcptaddr_bdb(struct search_res *res, struct config *cfg, char *rcptaddr,
         return RECORD_NOTFOUND;
     }
 
-    ret = search_rcptstr_bdb(res->rcpt_dbp, cfg->cf_rcptdbpath, rcptaddr, fqdn,
-			     res->onetime_pass, passlist, rdmpasslist);
+    ret = search_rcptstr_bdb(res, cfg, rcptaddr, fqdn, passlist, rdmpasslist);
     if (ret < 0) {
         free(searchaddr);
         return DB_ERROR;
@@ -614,7 +636,8 @@ search_rcptaddr_bdb(struct search_res *res, struct config *cfg, char *rcptaddr,
             free(passwd);
             passwd = NULL;
 
-            if (add_enclist(&passlist, subdomain, rcptaddr, tmppass) !=      0) {
+            if (add_enclist(cfg, &passlist, subdomain, rcptaddr, tmppass,
+                            res->ext_dbp, cfg->cf_extensiondbpath) != 0) {
                 free(searchaddr);
                 return MALLOC_ERROR;
             }
@@ -622,7 +645,9 @@ search_rcptaddr_bdb(struct search_res *res, struct config *cfg, char *rcptaddr,
             DEBUGLOG("Add keyword to passlist structure.(%s)", rcptaddr);
         } else {
             /* Add rdmpasslist list */
-            if (add_enclist(&rdmpasslist, OTHERKEYWORD, rcptaddr, res->onetime_pass) != 0) {
+            if (add_enclist(cfg, &rdmpasslist, subdomain, rcptaddr,
+                            res->onetime_pass, res->ext_dbp,
+                            cfg->cf_extensiondbpath) != 0) {
                 free(searchaddr);
                 return MALLOC_ERROR;
             }
@@ -643,15 +668,19 @@ search_rcptaddr_bdb(struct search_res *res, struct config *cfg, char *rcptaddr,
  * char 		*keyword
  * char 		*addr
  * char 		*passwd
+ * DB			*ext_dbp
+ * char			*extensiondbpath
  *
  * return value
  *       0		Success
  *       -1		Error
  */
 int
-add_enclist(struct rcptinfo ***head, char *keyword, char *addr, char *passwd)
+add_enclist(struct config *cfg, struct rcptinfo ***head, char *keyword, char *addr, char *passwd, DB *ext_dbp, char *extensiondbpath)
 {
     struct rcptinfo *p, *tmpp = NULL, *rcptinfo;
+    char *extension;
+    int ret;
 
     if (*head != NULL) {
         for (p = **head; p != NULL; p = p->Next) {
@@ -704,10 +733,43 @@ add_enclist(struct rcptinfo ***head, char *keyword, char *addr, char *passwd)
         return -1;
     }
 
+    if (cfg->cf_extensiondb && cfg->cf_extensiondb[0] != '\0') {
+        /* set extension */
+        ret = search_ext_bdb(ext_dbp, extensiondbpath, keyword, &extension);
+        if (ret == DB_ERROR) {
+            /* error ... */
+            log(ERR_SEARCH_EXT, "add_enclist", keyword);
+            free(rcptinfo->keyword);
+            free(rcptinfo->passwd);
+            free(rcptinfo);
+            return -1;
+        }
+
+        if (extension != NULL) {
+            if (check_validation_of_ext(cfg, extension) == FALSE) {
+                /* validation error: Ignore extensions */
+                rcptinfo->extension = NULL;
+            } else {
+                rcptinfo->extension = extension;
+            }
+        } else {
+            rcptinfo->extension = NULL;
+        }
+    }
+
     if (push_rcptlist(&(rcptinfo->rcptlist), addr) != 0) {
-        free(rcptinfo->keyword);
-        free(rcptinfo->passwd);
-        free(rcptinfo);
+        if (rcptinfo->keyword) {
+            free(rcptinfo->keyword);
+        }
+        if (rcptinfo->passwd) {
+            free(rcptinfo->passwd);
+        }
+        if (rcptinfo->extension) {
+            free(rcptinfo->extension);
+        }
+        if (rcptinfo) {
+            free(rcptinfo);
+        }
         return -1;
     }
     rcptinfo->Next = NULL;
@@ -993,14 +1055,16 @@ search_str_bdb(DB *dbp, const char *dbpath, char *search_str,
 {
     int ret;
 
-    /* search not_search_str */
-    ret = db_search(dbp, dbpath, not_search_str, match_passwd);
-    if (ret == DB_ERROR) {
-        return DB_ERROR;
-    }
-    if (ret == FOUND) {
-        DEBUGLOG("not_search_str found.(%s)", not_search_str);
-        return NOT_ENC;
+    if (not_search_str != NULL) {
+        /* search not_search_str */
+        ret = db_search(dbp, dbpath, not_search_str, match_passwd);
+        if (ret == DB_ERROR) {
+            return DB_ERROR;
+        }
+        if (ret == FOUND) {
+            DEBUGLOG("not_search_str found.(%s)", not_search_str);
+            return NOT_ENC;
+        }
     }
 
     /* search search_str */
@@ -1016,6 +1080,40 @@ search_str_bdb(DB *dbp, const char *dbpath, char *search_str,
     return RECORD_NOTFOUND;
 }
 
+/*
+ * search_ext_bdb()
+ *
+ * Get the extension.
+ *
+ * args:
+ *       DB   *dbp		pointer
+ *       char *dbpath		string
+ *       char *keyword 		string
+ *       char **extension	string pointer
+ *
+ * return value:
+ *       RECORD_NOTFOUND	not found
+ *       RECORD_FOUND		found
+ *       DB_ERROR		db error
+ */
+int
+search_ext_bdb(DB *dbp, const char *dbpath, char *keyword, char **extension)
+{
+    int ret;
+
+    *extension = NULL;
+
+    ret = db_search(dbp, dbpath, keyword, extension);
+    if (ret == DB_ERROR) {
+        return DB_ERROR;
+    } else if (ret == FOUND){
+        return RECORD_FOUND;
+    }
+
+    return RECORD_NOTFOUND;
+}
+
+
 /*
  * ldap_get_userdn()
  *
@@ -1490,3 +1588,38 @@ str_tolower(char *rcptaddr)
 
     return lower_addr;
 }
+
+/*
+ * check_validation_of_ext()
+ *
+ * struct config        *cfg
+ * char                 *extension
+ *
+ * return value
+ *       0              Success
+ *       -1             Error
+ */
+int
+check_validation_of_ext(struct config *cfg, char *extension)
+{
+    char ext_char[] = EXTENSION_CHAR;
+    int i;
+    char *ret = 0;
+    int ext_len = 0;
+    int zipfilename_len = 0;
+
+    ext_len = strlen(extension);
+    zipfilename_len = strlen(cfg->cf_zipfilename);
+    for (i = 0; i < ext_len; i++) {
+        ret = strchr(ext_char, extension[i]);
+        if (ret == NULL) {
+            log(INVALID_CHAR_EXTENSION, "check_validation_of_ext", extension);
+            return FALSE;
+        }
+    }
+    if (ext_len > 20 || zipfilename_len + ext_len > 256) {
+        log(INVALID_LEN_EXTENSION, "check_zip_extension", extension);
+        return FALSE;
+    }
+    return TRUE;
+}
