@@ -49,6 +49,7 @@
 #include "sendmail.h"
 #include "global.h"
 #include "mailzip_tmpl.h"
+#include "samma_policy.h"
 
 pthread_mutex_t gmime_lock = PTHREAD_MUTEX_INITIALIZER;
 
@@ -76,8 +77,15 @@ free_rcptinfo(struct rcptinfo *info)
     if (info != NULL) {
         for (p = info; p != NULL; p = tmpp) {
 	    tmpp = p->Next;
-	    free(p->passwd);
-	    free(p->keyword);
+	    if (p->passwd) {
+	        free(p->passwd);
+            }
+            if (p->keyword) {
+	        free(p->keyword);
+            }
+            if (p->extension) {
+	        free(p->extension);
+            }
 	    if (p->rcptlist != NULL) {
 	        free_rcptlist(p->rcptlist);
 	    }
@@ -150,7 +158,9 @@ delete_attachments_mail(SMFICTX *ctx, struct mailinfo *minfo,
     struct mailzip mz;
     struct rcptinfo *p;
     int i, ret;
-    char *temp = NULL, *rcptaddr;
+    char *rcptaddr;
+    struct rcptaddr *rdm_all = NULL;
+    struct rcptinfo tmp_rcptinfo;
 
     memset(&mz, 0, sizeof(struct mailzip));
 
@@ -181,38 +191,62 @@ delete_attachments_mail(SMFICTX *ctx, struct mailinfo *minfo,
 	}
 
 	/* add attach file */
-	if (add_attach_file(&mz, rdmpasslist) != 0) {
+	if (add_attach_file(cfg, &mz, rdmpasslist) != 0) {
 	    mailzip_clean(mz);
 	    return ZIP_CONVERT_FAILED;
 	}
+
+        /* 送信先リストの作成 */
+        for (p = rdmpasslist; p != NULL; p = p->Next) {
+            for (i = 0; (p->rcptlist + i)->rcpt_addr; i++) {
+                if (push_rcptlist(&rdm_all, (p->rcptlist + i)->rcpt_addr) != 0) {
+	            mailzip_clean(mz);
+                    free_rcptlist(rdm_all);
+	            return ZIP_CONVERT_FAILED;
+                }
+            }
+        }
+
+        tmp_rcptinfo.keyword = OTHERKEYWORD;
+        tmp_rcptinfo.keyword_len = strlen(OTHERKEYWORD);
+        tmp_rcptinfo.passwd = NULL;
+        tmp_rcptinfo.extension = NULL;
+        tmp_rcptinfo.rcptlist = rdm_all;
+        tmp_rcptinfo.Next = NULL;
+
     	/* send mail */
-	if (sendmail(&mz, cfg, rdmpasslist, from) != 0) {
+        if (sendmail(&mz, cfg, &tmp_rcptinfo, from) != 0) {
 	    mailzip_clean(mz);
+            free_rcptlist(rdm_all);
 	    return ZIP_CONVERT_FAILED;
         }
         /* unlink the temporary file */
         if (unlink(mz.encfilepath) == -1) {
             log(ERR_FILE_REMOVE, "delete_attachments_mail", unlink(mz.encfilepath));
             mailzip_clean(mz);
+            free_rcptlist(rdm_all);
             return ZIP_CONVERT_FAILED;
         }
 
-        for (i = 0; (rdmpasslist->rcptlist + i)->rcpt_addr != NULL; i++) {
-            rcptaddr = (rdmpasslist->rcptlist + i)->rcpt_addr;
+        for (i = 0; (rdm_all + i)->rcpt_addr != NULL; i++) {
+            rcptaddr = (rdm_all + i)->rcpt_addr;
             /* delete from envelope to */
             if (smfi_delrcpt(ctx, rcptaddr) == MI_FAILURE) {
                 log(ERR_DELETE_ADDRESS, "delete_attachment_mail", rcptaddr);
 	        mailzip_clean(mz);
+                free_rcptlist(rdm_all);
                 return ZIP_CONVERT_FAILED;
             }
         }
     } 
 
+    /* 送信先リストを開放 */
+    free_rcptlist(rdm_all);
     mailzip_clean(mz);
     return ZIP_CONVERT_SUCCESS;
 }
 
-extern int
+int
 zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
                  char *from, struct rcptinfo *passlist, struct rcptinfo *rdmpasslist)
 {
@@ -221,6 +255,9 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
     int i, ret, chk_replace, chk_replace_noconv;
     char *temp = NULL, *notify_message, *rcptaddr, *confirmation_message;
     char *org_p;
+    struct rcptaddr *rdm_noext = NULL;
+    struct rcptaddr *rdm_all = NULL;
+    struct rcptinfo tmp_rcptinfo;
 
     memset(&mz, 0, sizeof(struct mailzip));
 
@@ -245,12 +282,12 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
     if (passlist != NULL) {
 	for (p = passlist; p != NULL; p = p->Next) {
 	    /* encryption */
-	    if (convert_zip(cfg, &mz, &p) != 0) {
+	    if (convert_zip(cfg, &mz, p) != 0) {
 		mailzip_clean(mz);
 		return ZIP_CONVERT_FAILED;
 	    }
 	    /* add attach file */
-	    if (add_attach_file(&mz, p) != 0) {
+	    if (add_attach_file(cfg, &mz, p) != 0) {
 		mailzip_clean(mz);
 		return ZIP_CONVERT_FAILED;
 	    }
@@ -266,7 +303,8 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
 	        temp = tmpl_init();
 
 
-	        chk_replace = tmpl_tag_replace(temp, mz, p, from, &confirmation_message);
+	        chk_replace = tmpl_tag_replace(temp, mz, p, from,
+                                               &confirmation_message);
             switch(chk_replace) {
                 case ERROR:
                     p->passwd = org_p;	/* restore original password */
@@ -285,7 +323,7 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
                         return ZIP_CONVERT_FAILED;
                     }
 	        }
-            p->passwd = org_p;	/* restore original password */
+                p->passwd = org_p;	/* restore original password */
 	        tmpl_release(temp);
 
                 // set header field "References"
@@ -322,45 +360,125 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
                 }
             }
 	}
-    } 
+    }
 
     if (rdmpasslist != NULL) {
-	/* encryption */
-	if (convert_zip(cfg, &mz, &rdmpasslist) != 0) {
-	    mailzip_clean(mz);
-	    return ZIP_CONVERT_FAILED;
-	}
-	/* add attach file */
-	if (add_attach_file(&mz, rdmpasslist) != 0) {
-	    mailzip_clean(mz);
-	    return ZIP_CONVERT_FAILED;
-	}
+        /* ランダムパスワードで暗号化するメールの送信 */
+        for (p = rdmpasslist; p != NULL; p = p->Next) {
+            if (p->extension && p->extension[0] != '\0') {	
+                /* 拡張子が指定されている場合 */
+	        /* encryption */
+	        if (convert_zip(cfg, &mz, p) != 0) {
+	            mailzip_clean(mz);
+                    free_rcptlist(rdm_noext);
+                    free_rcptlist(rdm_all);
+	            return ZIP_CONVERT_FAILED;
+	        }
 
-    	/* send mail */
-	if (sendmail(&mz, cfg, rdmpasslist, from) != 0) {
-	    mailzip_clean(mz);
-	    return ZIP_CONVERT_FAILED;
-	}
+                /* add attach file */
+	        if (add_attach_file(cfg, &mz, p) != 0) {
+	            mailzip_clean(mz);
+                    free_rcptlist(rdm_noext);
+                    free_rcptlist(rdm_all);
+	            return ZIP_CONVERT_FAILED;
+	        }
 
-	temp = tmpl_init();
+    	        /* send mail */
+	        if (sendmail(&mz, cfg, p, from) != 0) {
+	            mailzip_clean(mz);
+                    free_rcptlist(rdm_noext);
+                    free_rcptlist(rdm_all);
+	            return ZIP_CONVERT_FAILED;
+	        }
+            } else {
+                /* 拡張子が指定されていない場合 */
+                /* まとめて送信するためのリストを作成 */
+                for (i = 0; (p->rcptlist + i)->rcpt_addr; i++) {
+                   if (push_rcptlist(&rdm_noext, (p->rcptlist + i)->rcpt_addr) != 0) {
+		        mailzip_clean(mz);
+                        free_rcptlist(rdm_noext);
+                        free_rcptlist(rdm_all);
+                       return ZIP_CONVERT_FAILED;
+                   }
+                }
+            }
 
-    chk_replace = tmpl_tag_replace(temp, mz, rdmpasslist, from, &notify_message);
-    switch(chk_replace) {
-        case ERROR:
-            tmpl_release(temp);
-            mailzip_clean(mz);
-            return ZIP_CONVERT_FAILED;
-        case CONVERT_ERROR:
-            // replace before convert string
-            chk_replace_noconv = tmpl_tag_replace_noconv(mz, rdmpasslist, from,
-                                                         &notify_message, cfg->cf_errmsgtemplatepath);
+            /* パスワード通知を送信するためのリストを作成 */
+            for (i = 0; (p->rcptlist + i)->rcpt_addr; i++) {
+                if (push_rcptlist(&rdm_all, (p->rcptlist + i)->rcpt_addr) != 0) {
+		    mailzip_clean(mz);
+                    free_rcptlist(rdm_noext);
+                    free_rcptlist(rdm_all);
+                    return ZIP_CONVERT_FAILED;
+                }
+            }
+        }
 
-            if (chk_replace_noconv == ERROR) {
-                tmpl_release(temp);
+        /* 拡張子が指定されていない宛先にメール送信 */
+        if (rdm_noext) {
+            tmp_rcptinfo.keyword = OTHERKEYWORD;
+            tmp_rcptinfo.keyword_len = strlen(OTHERKEYWORD);
+            tmp_rcptinfo.passwd = rdmpasslist->passwd;
+            tmp_rcptinfo.extension = NULL;
+            tmp_rcptinfo.rcptlist = rdm_noext;
+            tmp_rcptinfo.Next = NULL;
+
+            /* encryption */
+            if (convert_zip(cfg, &mz, &tmp_rcptinfo) != 0) {
                 mailzip_clean(mz);
+                free_rcptlist(rdm_noext);
+                free_rcptlist(rdm_all);
                 return ZIP_CONVERT_FAILED;
             }
-    }
+
+            /* add attach file */
+            if (add_attach_file(cfg, &mz, &tmp_rcptinfo) != 0) {
+                mailzip_clean(mz);
+                free_rcptlist(rdm_noext);
+                free_rcptlist(rdm_all);
+                return ZIP_CONVERT_FAILED;
+            }
+
+            /* send mail */
+            if (sendmail(&mz, cfg, &tmp_rcptinfo, from) != 0) {
+                mailzip_clean(mz);
+                free_rcptlist(rdm_noext);
+                free_rcptlist(rdm_all);
+                return ZIP_CONVERT_FAILED;
+            }
+            free_rcptlist(rdm_noext);
+        }
+
+        /* パスワード通知の受信者リスト用 rcptinfo 作成 */
+        tmp_rcptinfo.keyword = OTHERKEYWORD;
+        tmp_rcptinfo.keyword_len = strlen(OTHERKEYWORD);
+        tmp_rcptinfo.passwd = rdmpasslist->passwd;
+        tmp_rcptinfo.extension = NULL;
+        tmp_rcptinfo.rcptlist = rdm_all;
+        tmp_rcptinfo.Next = NULL;
+
+        /* パスワード通知メールの送信 */
+	temp = tmpl_init();
+
+        chk_replace = tmpl_tag_replace(temp, mz, &tmp_rcptinfo, from, &notify_message);
+        switch(chk_replace) {
+            case ERROR:
+                tmpl_release(temp);
+                mailzip_clean(mz);
+                free_rcptlist(rdm_all);
+                return ZIP_CONVERT_FAILED;
+            case CONVERT_ERROR:
+                // replace before convert string
+                chk_replace_noconv = tmpl_tag_replace_noconv(mz, &tmp_rcptinfo, from,
+                                                             &notify_message, cfg->cf_errmsgtemplatepath);
+
+                if (chk_replace_noconv == ERROR) {
+                    tmpl_release(temp);
+                    mailzip_clean(mz);
+                    free_rcptlist(rdm_all);
+                    return ZIP_CONVERT_FAILED;
+                }
+        }
 	tmpl_release(temp);
 
         // set header field "references"
@@ -369,6 +487,7 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
             if (ret == ERROR) {
                 free(notify_message);
                 mailzip_clean(mz);
+                free_rcptlist(rdm_all);
                 return ZIP_CONVERT_FAILED;
             }
         }
@@ -376,18 +495,22 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
 	if (passwd_sendmail(cfg, from, notify_message) != 0) {
 	    free(notify_message);
 	    mailzip_clean(mz);
+            free_rcptlist(rdm_all);
 	    return ZIP_CONVERT_FAILED;
 	}
 	free(notify_message);
-        for (i = 0; (rdmpasslist->rcptlist + i)->rcpt_addr != NULL; i++) {
-            rcptaddr = (rdmpasslist->rcptlist + i)->rcpt_addr;
+
+        for (i = 0; (rdm_all + i)->rcpt_addr; i++) {
+            rcptaddr = (rdm_all + i)->rcpt_addr;
             /* delete from envelope to */
             if (smfi_delrcpt(ctx, rcptaddr) == MI_FAILURE) {
                 log(ERR_DELETE_ADDRESS, "zip_convert_mail", rcptaddr);
 	        mailzip_clean(mz);
+                free_rcptlist(rdm_all);
                 return ZIP_CONVERT_FAILED;
             }
         }
+        free_rcptlist(rdm_all);
     } 
 
 #ifdef	DEBUG
@@ -427,7 +550,7 @@ zip_convert_mail(SMFICTX *ctx, struct mailinfo *minfo, struct config *cfg,
 #define CONTENTTYPE_ZIP		"application/zip"
 #define CONTENTDISPOSITION	"attachment"
 int
-add_attach_file(struct mailzip *mz, struct rcptinfo *rcpt)
+add_attach_file(struct config *cfg, struct mailzip *mz, struct rcptinfo *rcpt)
 {
     GMimeObject *object, *att;
     GMimePart *part;
@@ -437,6 +560,7 @@ add_attach_file(struct mailzip *mz, struct rcptinfo *rcpt)
     FILE *fp;
     int index, i;
     const char *filename;
+    char newfilename[FILE_NAME_MAX + 1];
 
     pthread_mutex_lock(&gmime_lock);
     object = g_mime_message_get_mime_part(mz->message);
@@ -512,7 +636,18 @@ add_attach_file(struct mailzip *mz, struct rcptinfo *rcpt)
     g_mime_object_set_disposition((GMimeObject *)part, CONTENTDISPOSITION);
 #endif
 
-    g_mime_part_set_filename(part, mz->attachfilename);
+    if (cfg->cf_extensiondb && cfg->cf_extensiondb[0] != '\0') {
+        if (rcpt->extension != NULL) {
+
+            replace_extension(newfilename, mz->attachfilename, rcpt->extension);
+            g_mime_part_set_filename(part, newfilename);
+        } else {
+            g_mime_part_set_filename(part, mz->attachfilename);
+        }
+    } else {
+        g_mime_part_set_filename(part, mz->attachfilename);
+    }
+
     // もしここでメモリリークが見つかった場合、修正例
     // (文字コードのテストをしっかりすべき）
     //g_mime_object_set_content_type_parameter((GMimeObject *)part, "filename", mz->attachfilename);
@@ -1202,6 +1337,52 @@ mk_deletelistfile(struct config *cfg, struct mailzip *mz)
 #define QOPTION		"-q"
 #define ANDCOMMAND	"&&"
 #define CURRENTDIR	"./"
+#define ZIP_FILE_PATH   "%s/%s"
+
+/*
+ * replace_extension
+ *
+ * Function
+ *      Recplace extension of file
+ *
+ * Argument
+ *      char * newfilename      path of zipfile after replaced new extension
+ *      char * filename         current filename
+ *      char * new_ext          new extension
+ *
+ * Return value
+ *      0       Normal end.
+ *      -1      Abormal end.
+ */
+int
+replace_extension(char *newfilename, char *filename, char *new_ext)
+{
+    char *p;
+    char data[PATH_MAX + 1];
+    int ext_len;
+    int name_len;
+    char *ext;
+
+    /* search dot */
+    ext = strrchr(filename, DOT);
+    if (!ext) {
+        /* if filename have not extension then add new extension to filename */
+        snprintf(newfilename, FILE_NAME_MAX + 1, "%s.%s", filename, new_ext);
+    } else {
+        /* length of extension */
+        ext_len = strlen(ext + 1);
+        /* 1 is length of DOT */
+        name_len = strlen(filename) - ext_len - 1;
+        p = strncpy(data, filename, name_len);
+        *(p + name_len) = '\0';
+
+        /* create new filename */
+        snprintf(newfilename, FILE_NAME_MAX + 1, "%s.%s", data, new_ext);
+    }
+
+    return 0;
+}
+
 /*
  * convert_zip
  *
@@ -1209,16 +1390,16 @@ mk_deletelistfile(struct config *cfg, struct mailzip *mz)
  *      ZIP conversion
  *
  * Argument
- *      struct config  *cfg	config structure
- *      char *tmpdir		temporary directory name
- *      char *passwd		encryption password
+ *      struct config  *cfg     config structure
+ *      char *tmpdir            temporary directory name
+ *      char *passwd            encryption password
  *
  * Return value
  *      0       Normal end.
  *      -1      Abormal end.
  */
 int
-convert_zip(struct config *cfg, struct mailzip *mz, struct rcptinfo **rcpt)
+convert_zip(struct config *cfg, struct mailzip *mz, struct rcptinfo *rcpt)
 {
     int    ret;
     pid_t  pid;
@@ -1235,15 +1416,15 @@ convert_zip(struct config *cfg, struct mailzip *mz, struct rcptinfo **rcpt)
      * make child process
      */
     if ((pid = fork()) == 0) {
-	/* child process */
-	zip_child(list, cfg->cf_zipcommandopt, mz->zipdir, (*rcpt)->passwd);
+        /* child process */
+        zip_child(list, cfg->cf_zipcommandopt, mz->zipdir, rcpt->passwd);
     } else if (pid > 0) {
         /* parent process */
         ret = waitpid(pid, &sts, WUNTRACED);
-	if (ret == -1) {
+        if (ret == -1) {
             log(ERR_WAIT_CHILD, "convert_zip", strerror(errno));
             return -1;
-	}
+        }
         if (WIFEXITED(sts)) {       /* return or exit */
             ret = WEXITSTATUS(sts); /* exit code */
             if (ret != 0) {
@@ -1255,9 +1436,9 @@ convert_zip(struct config *cfg, struct mailzip *mz, struct rcptinfo **rcpt)
             return -1;
         }
     } else {
-	/* make process failed */
+        /* make process failed */
         log(ERR_FORK_CREATE, "convert_zip", strerror(errno));
-	return -1;
+        return -1;
     }
 
     return (0);
